using System;
using System.Collections;
using System.Drawing;
using System.ComponentModel;
using System.Windows.Forms;

using Syncfusion.Windows.Forms.Grid;

namespace VirtGrid
{

	#region derived GridControl that has virtual overrides and uses a VirtGrid data

	public class VirtGrid : GridControl
	{
		public VirtData theData = null;

		public VirtGrid()
		{
			//add a sort header to the CellModels... 
			this.CellModels.Add("ColumnHeaderCell", new GridSortColumnHeaderCellModel(this.Model));

		}

		protected override void OnQueryCellInfo(Syncfusion.Windows.Forms.Grid.GridQueryCellInfoEventArgs e)
		{
			if(this.theData == null)
				return;
			base.OnQueryCellInfo(e);
			if(e.ColIndex > 0 && e.RowIndex > 0 && !e.Handled)
				e.Style.CellValue = this.theData[e.RowIndex, e.ColIndex];
		}

		protected override void OnSaveCellInfo(Syncfusion.Windows.Forms.Grid.GridSaveCellInfoEventArgs e)
		{
			if(this.theData == null)
				return;
			base.OnSaveCellInfo(e);
			if(e.ColIndex > 0 && e.RowIndex > 0 && !e.Handled)
			{
				this.theData[e.RowIndex, e.ColIndex] = (int) e.Style.CellValue;
				e.Handled = true;
//				if(this.theData[e.RowIndex, e.ColIndex] != null)
//				{
//					Console.WriteLine(this.Text + "-->" + this.theData[e.RowIndex, e.ColIndex].ToString());
//				}
//				else
//				{
//					Console.WriteLine(this.Text + "--> null");
//				}
			}
		
		}

		protected override void OnQueryRowCount(Syncfusion.Windows.Forms.Grid.GridRowColCountEventArgs e)
		{
			if(this.theData == null)
				return;

			base.OnQueryRowCount(e);
			if(!e.Handled)
			{
				e.Count = this.theData.RowCount;
				e.Handled = true;
			}
		}

		protected override void OnQueryColCount(Syncfusion.Windows.Forms.Grid.GridRowColCountEventArgs e)
		{
			if(this.theData == null)
				return;
			base.OnQueryRowCount(e);
			if(!e.Handled)
			{
				e.Count = this.theData.ColCount;
				e.Handled = true;
			}
		}

		public void SortCol(int col)
		{
			//Console.WriteLine("SortColumn {0}", col);
			Cursor.Current = Cursors.WaitCursor;
			
			if(this.CurrentCell.IsEditing)
				this.CurrentCell.EndEdit();
		
			GridStyleInfo style = new GridStyleInfo();
			style.Tag = "";

			//turn off the old sort arrow
			for(int col1 = 1; col1 <= this.ColCount; ++col1)
				if(col1 != col && this[0, col1].CellType == "ColumnHeaderCell")
				{
					this[0, col1].CellType = "Header";
					break;
				}

			//turn on new sort arrow
			this[0, col].CellType = "ColumnHeaderCell";

			if(this[0, col].HasTag)
			{
				if((ListSortDirection) this[0, col].Tag == ListSortDirection.Ascending)
					this[0, col].Tag = ListSortDirection.Descending;
				else
					this[0, col].Tag = ListSortDirection.Ascending;
			}
			else
			{
				this[0, col].Tag = ListSortDirection.Ascending;
			}

			this.theData.SortVirtData(col, (ListSortDirection)this[0, col].Tag);
			
			this.Refresh();
			Cursor.Current = Cursors.Default;
		}
//		protected override void OnCellDoubleClick(GridCellClickEventArgs e)
//		{e.Cancel = true;}
		protected override void OnMouseDown(MouseEventArgs e)  
		{	base.OnMouseDown(e);
			int row,col;
			Point pt = new Point(e.X, e.Y);
			if (e.Button == MouseButtons.Left)
			{
				if (e.Clicks == 1)
				{ 
					this.PointToRowCol(pt, out row, out col);
					SortCol(col);
				}
//				else if(e.Clicks == 1)
//				{	}
			}
		}  
	}
	#endregion

	#region wrapper for a virtual data store that supports sorting...

	public class VirtData
	{
		//datastore
		private int[,] values;
		private SortIndexList sortIndexes;
		
		
		public VirtData(int rows, int cols)
		{
			values = new int[rows, cols];
			this.cols = cols;

			sortIndexes = new SortIndexList(rows);
			for(int i = 0; i < rows; ++i)
				sortIndexes.Add(i);
		}

		//Row Count property - based on sortIndexes
		public int RowCount
		{
			get{return sortIndexes.Count;}
			//set{this.rows = value;}
		}

		//Column Count property
		private int cols;
		public int ColCount
		{
			get{return this.cols;}
			set{this.cols = value;}
		}

		//just returns the value at the absolute row, col without using SortIndexes
		//needed for sorting...
		public int GetRawValue(int row, int col)
		{
			return this.values[row - 1, col - 1];
		}

		//defines an indexer that uses SortIndexes to map the value look up
		public int this[int row, int col]
		{
			get
			{
				if(row <= this.RowCount && col <= this.cols && row > 0 && col > 0)
				{
					return this.values[sortIndexes[row - 1], col - 1];
				}
				else
					return 0;
			}

			set
			{
				if(row <= this.RowCount && col <= this.cols && row > 0 && col > 0)
				{
					this.values[sortIndexes[row - 1], col - 1] = value;
				}				
			}
		}


		//method that sorts a column in the data object
		public void SortVirtData(int colIndex, ListSortDirection sortDir)
		{
			DataComparer.column = colIndex;
			DataComparer.data = this;
			DataComparer.sortDir = sortDir;

			this.sortIndexes.Sort(new DataComparer());
		}

		
		//just creates some dummy data
		public static void InitDummyData(VirtData data)
		{
			Random r = new Random();
			for(int i = 0; i <= data.RowCount; ++ i)
				for(int j = 0; j <= data.ColCount; ++j)
				{
					if(j == 1)
						data[i, j] = i;
					else if(j == 3)
						data[i, j] = 10 * data[i, j-1];
					else
						data[i, j] = r.Next(10000);
				}
		}

	}

	#endregion

	#region IComparer object that orders the sortindex arraylist based on values in particular column

	public class DataComparer : IComparer
	{
		public static ListSortDirection sortDir = ListSortDirection.Ascending;
		public static VirtData data = null;
		public static int column = -1;

		public int Compare(object x, object y)
		{
			int val = 0;
			if(x == null)
				val = -1;
			else if(y == null)
				val = 1;
			else if(x != null || y != null)
				val = data.GetRawValue((int)x+1, column) - data.GetRawValue((int)y+1, column);

			return (sortDir == ListSortDirection.Ascending) ? val : -val;
		}
		

	}

	#endregion

	#region SortIndexList - arraylist of int indexes

		//used as int-typed arraylist
		public class SortIndexList : ArrayList
		{
			public SortIndexList(int rows) : base(rows)
			{

			}

			//eg. use for ints
			public new int this[int row]
			{
				get {return (int)base[row];}
				set {base[row] = value;}
			}


		}
	#endregion

	}
